pico-8 cartridge // http://www.pico-8.com
version 7
__lua__
function _init()
  flip_display=false
  flash=false
  display = 0
  t = 0
  c_col=0
  note = 0
  bar = 0
  bpm = 128
  num_b = 1
  text = "PRESS b TO jump"
  a = text_table(text)
  cls()
end

function text_table(string)
  tab={}
  for i=0, #string do
    char = sub(string ,i, i)
    add(tab, char)
  end
  return tab
end

function _update()
  -- time flow (use with sin/cos)
  if (t>=1) t=0
  if (t<1) t+=0.01
  -- bpm time flow
  fpb = flr((30*60)/bpm)
  if (note<fpb) note+=1
  if (note>=fpb) note=0 bar+=1
  if (bar>=4) bar=0
  v = (note + (bar*fpb)) / (fpb * 4)
  cls()
end

function draw_grid_background(c1,c2,c3)
  for j=0, 16 do
    for k=0, 32 do
      pset(k*2+rnd(64), j*8, c1)
      pset(j*8+rnd(4) ,k*4 + rnd(2),c2)
      pset(j*8,k*2+rnd(64),c3)
    end
  end
end

function draw_oscillating_text(table, y, ay)
  k=0
  width = (((#table*5)-128)/2)*(cos(2*v)-1)
  for c in all(table) do
    print(c, k*5  + width -1, y + ay*sin(t+1/(#table)*k)-1 , 0)
    print(c, k*5 + width, y + ay*sin(t+1/(#table)*k) , rnd(8)+8)
    k+=1
  end
end

function _draw()

  -- screen flash and random grid background with tempo
  if (flash) then
    if (note==0) rectfill(0,0,127,127,7)
    if (note==2) draw_grid_background(10,12,14)
    if (note==4) draw_grid_background(7,8,16)
    if (note==6) draw_grid_background(8,14,11)
    if (note==8) draw_grid_background(9,13,12)
  end

  -- diagonal graphics?
  if ((display>=4)and (note%4)==0) then
    x1 = 0
    y1= 64 +32*sin(v)
    for i=0,num_b do
      y2 = 128*(i%2) + (i*4) * ((-1)^(i%2))
      col = rnd(16)
      line(x1,y2, 64, y1, col)
      line(128-x1,y2, 64, y1, col)
    end
  end

  -- sine deltas + balls
  if ((display==2) or (display==3) or (display==6) or (display==7)) then
    x1= 64+ 32*cos(t)
    y1= 64+ 16*sin(t)
    if((note%4)==0) circfill(x1,y1,(8*cos(t/2))^2, rnd(8)+8)
    for i=0,num_b do
      x2 = i*(128/num_b) + (i-num_b/2)*5*cos(v)
      y2 = sin((i+1)*sin(v)/num_b)*32+64
      c_col = note*bar
      if (c_col>16) c_col=rnd(16)
      line(x2,y2, x1, y1, c_col)
      circfill(x2,y2, 4, c_col)
    end
  end

  -- oscillating text plus shadow
  if ((display%2)==1) draw_oscillating_text(a, 15, 5)

  -- btn controls
  -- bpm down
  if ((btnp(4)) and (btn(0) and (bpm > 40))) bpm-=1
  -- bpm up
  if ((btnp(4)) and (btn(1)) and (bpm < 255)) bpm+=1
  -- number of sine wave deltas up (up to 36)
  if ((btnp(4)) and (btn(2)) and (num_b<32)) num_b+=1
  if ((btnp(4)) and (btn(3)) and (num_b>1)) num_b-=1

  -- BPM SHOW with up + down
  if (btn(0) and btn(1)) print(bpm, 5,5,7)

  -- MIRROR MODES!
  if ((btnp(4)) and (btnp(5)) and (flip_display)) then -- restore default screen
    poke(0x5f2c, 0)
    flip_display=false
  else --change screen view with random mirror mode
    if ((btnp(4)) and (btnp(5)) and (not flip_display)) then
      poke(0x5f2c, flr(rnd(3))+5)
      flip_display=true
    end
  end
  -- to flash or not to flash
  if (btnp(0) and btnp(5)) flash= not flash

  -- (0=none, 1=text, 2=sine deltas, 3=both)
  -- (4=diagonal, 5=text+diagonal, 6=deltas+diagonal, 7=all)
  if (btnp(5) and btnp(2)) display = display+1
  if (btnp(5) and btnp(3)) display = display-1
  if (display >= 8) display = 0
  if (display < 0) display = 7

  -- display a rectangle
  rect(0,0,127,127,7)

end
